﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Text;
using System.Xml;
using System.IO;
using System.Security.Cryptography.X509Certificates;


namespace dk.looksharp.test.btshttpreceive
{

    class HttpsClient : WebClient
    {
        public HttpStatusCode StatusCode { get; set; }
        public string StatusCodeDescription { get; set; }
        public DateTime LastModified { get; set; } 

        public string Response { get; set; }
        public X509Certificate2 Certificate { get; set; }

        // Override WebRequest to add certificate to the WebRequest object
        protected override WebRequest GetWebRequest(Uri address)
        {
            HttpWebRequest request = (HttpWebRequest)base.GetWebRequest(address);
            request.ClientCertificates.Add(this.Certificate);
            return request;
        }

        // Intercept the WebRequest object to get response data from host
        protected override WebResponse GetWebResponse(WebRequest request)
        {
            this.StatusCode = HttpStatusCode.Accepted;
                
            // Get response stream from web request
            HttpWebResponse _webResponse = (HttpWebResponse)request.GetResponse();

            this.StatusCode = _webResponse.StatusCode;
            this.StatusCodeDescription = _webResponse.StatusDescription;
            this.LastModified = _webResponse.LastModified;

            // Is the stream readable?
            if (this.StatusCode == HttpStatusCode.Accepted && _webResponse.GetResponseStream().CanRead)
            {
                using (StreamReader _reader = new StreamReader(_webResponse.GetResponseStream(), true))
                {
                    // Get stream content
                    this.Response = _reader.ReadToEnd();
                }
            }
            return base.GetWebResponse(request);
        }
    }

    class Program
    {

        static X509Certificate2 GetClientCertificateFromThumbPrint(string thumbPrint)
        {

            if (string.IsNullOrEmpty(thumbPrint))
                throw new ArgumentException(
                    String.Format("Error: Thumbprint is empty"), "ThumbPrint"
                    );

            X509Store _store = new X509Store(
                StoreName.AddressBook, StoreLocation.CurrentUser
                );

            // Try to open the store.
            _store.Open(OpenFlags.ReadOnly);
            // Find the certificate that matches the thumbprint.
            X509Certificate2Collection _certs = _store.Certificates.Find(
                X509FindType.FindByThumbprint, thumbPrint, false
                );
            _store.Close();

            // Check to see if our certificate was added to the collection. If no, throw an error, if yes, create a certificate using it.
            if (_certs.Count == 0)
            {
                throw new ArgumentException(
                    String.Format("Error: No certificate found containing thumbprint: {0}", thumbPrint), "ThumbPrint"
                    );
            }
            return _certs[0];
        }

        // Main program
        static void Main(string[] args)
        {
            byte[] _responseHeaders = null;
            string _clientResponse = string.Empty;


            X509Certificate2 _certificate = null;
            try
            {
                string _thumbPrint = "1a 2b 3c 4d 5e 6f a1 b2 c3 d4 e5 f6 1a 2b 3c 4d 5e 6f a1 b2";
                _certificate = GetClientCertificateFromThumbPrint(_thumbPrint);
            }
            catch (Exception exc)
            {
                Console.WriteLine(exc);
                throw new Exception("Certificate exception", exc);
            }

            // Url to BizTalk Http Receive ISAPI dll
            Uri _address = new Uri("https://biztalk.test.invalid/test/BTSHTTPReceive.dll");
            // Load file from disk or other source
            string _filePath = @"d:\data\test\Order.4b496d3c-6214-495b-bbce-1850516b6cf9.xml";

            try
            {
                using (HttpsClient _client = new HttpsClient())
                {
                    _client.Certificate = _certificate;
                    using (Stream _stream = _client.OpenWrite(_address))
                    {
                        using (FileStream _fileStream = new FileStream(_filePath, FileMode.Open, FileAccess.Read))
                        {
                            _fileStream.CopyTo(_stream);
                        }
                        _stream.Flush();
                    }

                    Console.WriteLine("{0} at {1:F}", _client.StatusCodeDescription, _client.LastModified);
                    Console.WriteLine("");

                    _responseHeaders = _client.ResponseHeaders.ToByteArray();
                    _clientResponse = _client.Response;

                }
            }
            catch (WebException exc)
            {
                Console.WriteLine(exc);
                throw new Exception("WebClient exception", exc);
            }

            finally
            {
                if (!string.IsNullOrEmpty(_clientResponse))
                    Console.WriteLine(_clientResponse);

                if (_responseHeaders != null && _responseHeaders.Length > 0)
                    // Decode and display the response.
                    Console.WriteLine(Encoding.ASCII.GetString(_responseHeaders));
            }
        }
    }
}
